# ***************************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# ***************************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# ***************************************************************************
import json
import nsx_logging
import sys

from cli.commands.cli_command_provider import CliCommandProvider
from cli.utils.nestdb_utils import invoke_nestdb_command, query_type
from cli.utils.ipaddr_utils import mac_int_to_string, ip_int_to_string
from cli.utils.error_code import ErrorCode as ec

TRANSPORT_NODE_CONFIG_MSG = "vmware.nsx.nestdb.TransportNodeConfigMsg"
LOG_SWITCH_CONFIG_MSG = "vmware.nsx.nestdb.LogSwitchConfigMsg"
LOG_SWITCH_PORT_CONFIG_MSG = "vmware.nsx.nestdb.LogSwitchPortConfigMsg"
LOG_SWITCH_STATE_MSG = "vmware.nsx.nestdb.LogSwitchStateMsg"
LOG_SWITCH_FIB_MSG = "vmware.nsx.nestdb.LogSwitchFibMsg"
PORT_MIRROR_SESSION_MSG = "vmware.nsx.nestdb.PortMirrorSessionMsg"
IPFIX_CONFIG_MSG = "vmware.nsx.nestdb.IpfixConfigurationMsg"

lg = nsx_logging.getLogger("cli.commands.lcp_common.nsx_proxy.logical_switch_l2")


class NestDBL2Convertor():
    @staticmethod
    def mac_cache_convertor(mac):
        entry = {}
        if "mac" in mac and "mac" in mac["mac"]:
            entry["vm_mac"] = mac_int_to_string(mac["mac"]["mac"])
        if "vtep_ip" in mac:
            if "ipv4" in mac["vtep_ip"]:
                entry["vtep_ip"] = ip_int_to_string(mac["vtep_ip"]["ipv4"])
            if "ipv6" in mac["vtep_ip"]:
                entry["vtep_ipv6"] = mac["vtep_ip"]["ipv6"]
        if "vtep_mac" in mac and "mac" in mac["vtep_mac"]:
            entry["vtep_mac"] = mac_int_to_string(mac["vtep_mac"]["mac"])
        return entry

    @staticmethod
    def vtep_cache_convertor(vtep):
        entry = {}
        if "vtep_ip" in vtep:
            if "ipv4" in vtep["vtep_ip"]:
                entry["vtep_ip"] = ip_int_to_string(vtep["vtep_ip"]["ipv4"])
            if "ipv6" in vtep["vtep_ip"]:
                entry["vtep_ipv6"] = vtep["vtep_ip"]["ipv6"]
        if "segment_id" in vtep:
            if "ipv4" in vtep["segment_id"]:
                entry["subnet_ip"] = ip_int_to_string(vtep["segment_id"]["ipv4"])
            if "ipv6" in vtep["segment_id"]:
                entry["subnet_ip_v6"] = vtep["segment_id"]["ipv6"]
        if "vtep_label" in vtep and "label" in vtep["vtep_label"]:
            entry["label"] = vtep["vtep_label"]["label"]
        if "vtep_mac" in vtep and "mac" in vtep["vtep_mac"]:
            entry["vtep_mac"] = mac_int_to_string(vtep["vtep_mac"]["mac"])
        return entry

    @staticmethod
    def arp_cache_convertor(arp):
        entry = {}
        if "ip" in arp:
            if "ipv4" in arp["ip"]:
                entry["vm_ip"] = ip_int_to_string(arp["ip"]["ipv4"])
            if "ipv6" in arp["ip"]:
                entry["vm_ipv6"] = arp["ip"]["ipv6"]
        if "mac" in arp and "mac" in arp["mac"]:
            entry["vm_mac"] = mac_int_to_string(arp["mac"]["mac"])
        return entry

    @staticmethod
    def get_log_switch_msg(nestdb_schema, vni):
        rc, out, err = query_type(LOG_SWITCH_CONFIG_MSG,
                                  [("vni", "EQUALS", int(vni))])
        if rc != 0:
            lg.error("MSG_CLI_UNEXPECTED_ERROR\n%s",
                     err, error_code=ec.OPERATION_FAILED)
            return (-1, None, "invalid command")

        lsconfig = json.loads(out)
        length = len(lsconfig)
        if length != 1 or "id" not in lsconfig[0]:
            return (0, None, "not exist")

        ls_id = json.dumps(lsconfig[0]["id"])

        cmd = ["query %s {\"id\":{\"self\":{\"op\":\"EQUALS\"}}} {\"id\":%s}"
               % (nestdb_schema, ls_id)]
        return invoke_nestdb_command(cmd)

    @staticmethod
    def parse_ls_msg(nestdb_schema, vni):
        rc, rv, err = NestDBL2Convertor.get_log_switch_msg(nestdb_schema, vni)
        if rc != 0:
            return (rc, rv, err)

        if rv is None:
            return (0, None, "not exist")

        resp = json.loads(rv)
        if "result" in resp:
            out = [v["value"] for v in resp["result"]]
            return (0, out, None)
        else:
            return (0, None, "not exist")

    @staticmethod
    def get_ls_cache_by_key(nestdb_schema, vni, key):
        _L2_CONVERTOR_MAPPING = {
            'mac':  NestDBL2Convertor.mac_cache_convertor,
            'vtep': NestDBL2Convertor.vtep_cache_convertor,
            'ip':  NestDBL2Convertor.arp_cache_convertor,
        }

        rc, rv, err = NestDBL2Convertor.parse_ls_msg(nestdb_schema, vni)
        if rc != 0:
            return None

        entry_list = []

        if rv is not None:
            for val in rv:
                if key in val:
                    for entry in val[key]:
                        entry_list.append(_L2_CONVERTOR_MAPPING[key](entry))

        return entry_list

    @staticmethod
    def clear_desired_config():
        desired_config_list = [IPFIX_CONFIG_MSG, PORT_MIRROR_SESSION_MSG, \
                               LOG_SWITCH_PORT_CONFIG_MSG, LOG_SWITCH_CONFIG_MSG, \
                               TRANSPORT_NODE_CONFIG_MSG]
        for config_schema in desired_config_list:
            cmd = ["flush %s" % config_schema]
            rc, rv, err = invoke_nestdb_command(cmd)
            if rc != 0:
               return None
        return "OK\n"


class GetLSLocalMAC(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        local_mac = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                          vni, 'mac')
        if local_mac is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"
        return "OK\n\n%s" % json.dumps(local_mac)


class GetLSRemoteMAC(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        remote_mac = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_FIB_MSG,
                                                           vni, 'mac')
        if remote_mac is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        local_mac = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                          vni, 'mac')
        if local_mac is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        for elem in local_mac:
            if elem in remote_mac:
                remote_mac.remove(elem)

        return "OK\n\n%s" % json.dumps(remote_mac)


class GetLSLocalVTEP(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        local_vtep = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                           vni, 'vtep')
        if local_vtep is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"
        return "OK\n\n%s" % json.dumps(local_vtep)


class GetLSRemoteVTEP(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        remote_vtep = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_FIB_MSG,
                                                            vni, 'vtep')
        if remote_vtep is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        local_vtep = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                            vni, 'vtep')
        if local_vtep is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        for elem in local_vtep:
            if elem in remote_vtep:
                remote_vtep.remove(elem)

        return "OK\n\n%s" % json.dumps(remote_vtep)


class GetLSLocalARP(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        local_arp = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                          vni, 'ip')
        if local_arp is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"
        return "OK\n\n%s" % json.dumps(local_arp)


class GetLSRemoteARP(CliCommandProvider):
    def __call__(self, cmd_list):
        vni = cmd_list[4]
        remote_arp = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_FIB_MSG,
                                                           vni, 'ip')
        if remote_arp is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        local_arp = NestDBL2Convertor.get_ls_cache_by_key(LOG_SWITCH_STATE_MSG,
                                                           vni, 'ip')
        if local_arp is None:
            return "MSG_NSX_PROXY_CACHE_TABLE_FAILED"

        for elem in local_arp:
            if elem in remote_arp:
                remote_arp.remove(elem)

        return "OK\n\n%s" % json.dumps(remote_arp)


class NsxProxyDelTransportNode(CliCommandProvider):
    def __call__(self, cmd_list):
        response = NestDBL2Convertor.clear_desired_config()
        if response is None:
            return "MSG_NSX_PROXY_TRANSPORT_NODE_DELETE_FAILED"

        return "OK\n\n%s" % json.dumps(response)

