# **********************************************************************
# Copyright 2017 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
from copy import copy
import nsx_logging

from cli.utils.constants import CENTRAL_CLI_END_TOKEN
from cli.utils.constants import CENTRAL_CLI_START_TOKEN


lg = nsx_logging.getLogger(__name__)


class CentralCLIException(Exception):
    def __init__(self, msg):
        super(CentralCLIException, self).__init__(msg)


def is_central_cli_command(cmd_list):
    """Return whether given command is a Central CLI command"""
    # TODO 'on' can be loaded from CentralCLI descriptor, instead of hardcoding
    try:
        return cmd_list[0] == CENTRAL_CLI_START_TOKEN
    except:
        return False


def get_central_cli_nodes(cmd_list):
    """Return the hosts from a Central CLI command

    ['on', 'host1', 'host2', CENTRAL_CLI_END_TOKEN, 'get', 'hostname'] =>
    ['host1', 'host2']
    """
    if not is_central_cli_command(cmd_list):
        return []

    try:
        end_token_index = cmd_list.index(CENTRAL_CLI_END_TOKEN)
        return cmd_list[1:end_token_index]  # 0th item is 'on'
    except ValueError:  # If command doesn't contain CENTRAL_CLI_END_TOKEN
        return cmd_list[1:]  # return till end


def get_central_cli_user_input(cmd_list):
    """Return the command part from a Central CLI command

    ['on', 'host1', 'host2', CENTRAL_CLI_END_TOKEN, 'get', 'hostname'] =>
    ['get', 'hostname']
    """
    if not is_central_cli_command(cmd_list):
        return []

    try:
        end_token_index = cmd_list.index(CENTRAL_CLI_END_TOKEN)
        return cmd_list[end_token_index+1:]  # 0th item is 'on'
    except ValueError:   # If command doesn't contain CENTRAL_CLI_END_TOKEN
        return []


def get_cmd_list_path_tuple(cmd_list):
    """Format user input in cmd_list format into how command descriptors
       declare its path_tuple.

       In Dropkick cmd_list and Descriptor.path_tuple is the same except
       when the user enters a Central CLI command. This is due to the
       Central Cli's multi-arg support <registered-node-uuid> (a single
       CliArgument that accepts mutiple space delimited node UUIDs
       followed by the literal "exec"). This method collapses
       <registered-node-uuid> values into a single string value so that
       the resulting cmd_list can be compared with Descriptor.path_tuple
       on a per index basis.

       TODO: if additional multi-arg CliAruguments are added in future
       relase, this method needs to support those too.


    Non-Central CLI example:
    ['get', 'hostname'] =>
    ['get', 'hostname']

    Central Cli example:
    ['on', 'host1', 'host2', CENTRAL_CLI_END_TOKEN, 'get', 'hostname'] =>
    ['on' 'host1 host2 CENTRAL_CLI_END_TOKEN', 'get', 'hostname']
    """
    if not is_central_cli_command(cmd_list):
        return copy(cmd_list)
    specified_nodes = get_central_cli_nodes(cmd_list)
    if not specified_nodes:
        return copy(cmd_list)
    if len(cmd_list) > (len(specified_nodes) + 1):
        specified_nodes.append(cmd_list[len(specified_nodes) + 1])
    result = [cmd_list[0], " ".join(specified_nodes)]
    result.extend(get_central_cli_user_input(cmd_list))
    return result
