#!/usr/bin/env python
# **********************************************************************
# Copyright 2016 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************

'''Feature switch run-time library for python'''

import os
import sys
import logging


 # pylint:disable=C0103
logger = logging.getLogger(__name__)
_switches = {}
LINUX_LOCAL_CONF = '/opt/vmware/nsx/featureswitch/conf/switches.yaml'
OVERRIDE_CONF = 'VMware\\NSX\\Data\\FeatureSwitch\\switches.yaml'


def str2bool(val):
    if val == 'True' or val == 'true':
        return True
    elif val == 'False' or val == 'false':
        return False
    else:
        logger.error('ERROR: Invalid switch value: %s', val)
        sys.exit(1)


def read_local_config(conf):
    '''Read local switch config file'''
    with open(conf, 'r') as stream:
        for line in stream:
            feature, option = line.split(':')
            feature = feature.strip()
            if feature == 'AllowOverrideOnDeploy':
                continue
            option = str2bool(option.strip(' \n'))
            if feature not in _switches:
                logger.error(('ERROR: feature %s is not in current'
                              ' feature list'), feature)
                sys.exit(1)
            _switches[feature] = option


def feature_switch_init():
    '''Feature switch Initilization'''
    _switches['MigrateToCvds'] = False
    _switches['InterSiteMeshScaleOptimization'] = True
    _switches['Lb'] = True
    _switches['PublicCloudAzure'] = True
    _switches['Waf'] = False
    _switches['AdvancedLoadBalancerControllerNodeDeployment'] = True
    _switches['HostProfileSupport'] = True
    _switches['UfoDevWorkInProgressForL2'] = False
    _switches['ProtonToUfoMessagingServiceDBSync'] = False
    _switches['PublicCloud'] = True
    _switches['EdgeLrportArpSuppression'] = False
    _switches['BITWBridge'] = False
    _switches['BgpLearnedAdvertisedRoutesAPIs'] = True
    _switches['ServiceInsertion'] = True
    _switches['Nioc'] = True
    _switches['CentosBareMetal'] = False
    _switches['MicrosoftHyperV'] = False
    _switches['SwitchingUPMProfile'] = True
    _switches['Container'] = True
    _switches['SfhcVerticalRegisterWithMessagingPolicyUFO'] = True
    _switches['VifOperationEnhancement'] = True
    _switches['PolicyALB'] = True
    _switches['EnsOverlay'] = True
    _switches['DirectoryService'] = True
    _switches['EsxFailcode'] = False
    _switches['MultiTenancy'] = True
    _switches['PolicyLb'] = True
    _switches['IpDiscoveryGroupBinding'] = True
    _switches['SwitchingVerticalMessagingUFO'] = True
    _switches['Abstractions'] = False
    _switches['UnrestrictedSKU'] = True
    _switches['Example'] = False
    _switches['ApiRealizationTracking'] = True
    _switches['ControllerMgmtVerticalMessagingUFO'] = True
    _switches['EdgeServiceInContainer'] = True
    _switches['EdgeVerticalMessagingPolicyUFO'] = True
    _switches['EdgeSsl'] = True
    _switches['InventoryContainer'] = True
    _switches['GeneralSecuritySettingsProfile'] = False
    _switches['L2EphemeralLogicalPortSynchronousDeletion'] = False
    _switches['VmcTraceflowEnabled'] = True
    _switches['MultiplePNic'] = False
    _switches['UnifiedProfileManagementSupportNSGroup'] = True
    _switches['PBR'] = True
    _switches['FederationConfigOnboarding'] = True
    _switches['Mpls'] = False
    _switches['NsxVPolicy'] = False
    _switches['RealizedBindingArpSuppression'] = True
    _switches['BmsVerticalMessagingUFO'] = True
    _switches['IntrusionServices'] = True
    _switches['ProximityRouting'] = True
    _switches['IpDiscovery'] = True
    _switches['NsxtOnVds'] = True
    _switches['NetworkEncryption'] = True
    _switches['NsxtOnVdsStatelessEsx'] = True
    _switches['DirectoryServiceSyncNodes'] = False
    _switches['NetworkEncryptionOverride'] = False
    _switches['LSRealizationViaCCP'] = True
    _switches['Ospf'] = True
    _switches['MpClustering'] = True
    _switches['VirtualServerResourceLimits'] = False
    _switches['NsxDashboardWidgetConfig'] = True
    _switches['PaceClusterNodeDeployment'] = True
    _switches['IdentityFirewall'] = True
    _switches['URLFiltering'] = True
    _switches['TNUpgradeToVds'] = True
    _switches['NsxUens'] = True
    _switches['ServiceConfig'] = True
    _switches['AutoTn'] = True
    _switches['InventoryApplications'] = False
    _switches['PcgUnifiedMessaging'] = True
    _switches['SRMColdMigration'] = True
    _switches['UnifiedMessaging'] = True
    _switches['InventoryEdgeInstall'] = True
    _switches['PolicyFirewallIdentityConfig'] = True
    _switches['LambdaQuerying'] = True
    _switches['CustomRole'] = True
    _switches['LsRealizationViaVC'] = False
    _switches['DestinationFiltering'] = False
    _switches['OIDC'] = False
    if sys.platform == 'win32':
        program_path = os.getenv('ProgramData')
        if program_path == None:
            logger.error('ERROR: missing environment variable ProgramData')
            sys.exit(1)
        local_conf = os.path.join(program_path, OVERRIDE_CONF)
    else:
        local_conf = LINUX_LOCAL_CONF
    if os.path.isfile(local_conf):
        read_local_config(local_conf)


def is_migrate_to_cvds_enabled():
    '''Returns feature migrate_to_cvds switch value'''
    return _switches['MigrateToCvds']


def is_inter_site_mesh_scale_optimization_enabled():
    '''Returns feature inter_site_mesh_scale_optimization switch value'''
    return _switches['InterSiteMeshScaleOptimization']


def is_lb_enabled():
    '''Returns feature lb switch value'''
    return _switches['Lb']


def is_public_cloud_azure_enabled():
    '''Returns feature public_cloud_azure switch value'''
    return _switches['PublicCloudAzure']


def is_waf_enabled():
    '''Returns feature waf switch value'''
    return _switches['Waf']


def is_advanced_load_balancer_controller_node_deployment_enabled():
    '''Returns feature advanced_load_balancer_controller_node_deployment switch value'''
    return _switches['AdvancedLoadBalancerControllerNodeDeployment']


def is_host_profile_support_enabled():
    '''Returns feature host_profile_support switch value'''
    return _switches['HostProfileSupport']


def is_ufo_dev_work_in_progress_for_l_2_enabled():
    '''Returns feature ufo_dev_work_in_progress_for_l_2 switch value'''
    return _switches['UfoDevWorkInProgressForL2']


def is_proton_to_ufo_messaging_service_d_b_sync_enabled():
    '''Returns feature proton_to_ufo_messaging_service_d_b_sync switch value'''
    return _switches['ProtonToUfoMessagingServiceDBSync']


def is_public_cloud_enabled():
    '''Returns feature public_cloud switch value'''
    return _switches['PublicCloud']


def is_edge_lrport_arp_suppression_enabled():
    '''Returns feature edge_lrport_arp_suppression switch value'''
    return _switches['EdgeLrportArpSuppression']


def is_b_i_t_w_bridge_enabled():
    '''Returns feature b_i_t_w_bridge switch value'''
    return _switches['BITWBridge']


def is_bgp_learned_advertised_routes_a_p_is_enabled():
    '''Returns feature bgp_learned_advertised_routes_a_p_is switch value'''
    return _switches['BgpLearnedAdvertisedRoutesAPIs']


def is_service_insertion_enabled():
    '''Returns feature service_insertion switch value'''
    return _switches['ServiceInsertion']


def is_nioc_enabled():
    '''Returns feature nioc switch value'''
    return _switches['Nioc']


def is_centos_bare_metal_enabled():
    '''Returns feature centos_bare_metal switch value'''
    return _switches['CentosBareMetal']


def is_microsoft_hyper_v_enabled():
    '''Returns feature microsoft_hyper_v switch value'''
    return _switches['MicrosoftHyperV']


def is_switching_u_p_m_profile_enabled():
    '''Returns feature switching_u_p_m_profile switch value'''
    return _switches['SwitchingUPMProfile']


def is_container_enabled():
    '''Returns feature container switch value'''
    return _switches['Container']


def is_sfhc_vertical_register_with_messaging_policy_u_f_o_enabled():
    '''Returns feature sfhc_vertical_register_with_messaging_policy_u_f_o switch value'''
    return _switches['SfhcVerticalRegisterWithMessagingPolicyUFO']


def is_vif_operation_enhancement_enabled():
    '''Returns feature vif_operation_enhancement switch value'''
    return _switches['VifOperationEnhancement']


def is_policy_a_l_b_enabled():
    '''Returns feature policy_a_l_b switch value'''
    return _switches['PolicyALB']


def is_ens_overlay_enabled():
    '''Returns feature ens_overlay switch value'''
    return _switches['EnsOverlay']


def is_directory_service_enabled():
    '''Returns feature directory_service switch value'''
    return _switches['DirectoryService']


def is_esx_failcode_enabled():
    '''Returns feature esx_failcode switch value'''
    return _switches['EsxFailcode']


def is_multi_tenancy_enabled():
    '''Returns feature multi_tenancy switch value'''
    return _switches['MultiTenancy']


def is_policy_lb_enabled():
    '''Returns feature policy_lb switch value'''
    return _switches['PolicyLb']


def is_ip_discovery_group_binding_enabled():
    '''Returns feature ip_discovery_group_binding switch value'''
    return _switches['IpDiscoveryGroupBinding']


def is_switching_vertical_messaging_u_f_o_enabled():
    '''Returns feature switching_vertical_messaging_u_f_o switch value'''
    return _switches['SwitchingVerticalMessagingUFO']


def is_abstractions_enabled():
    '''Returns feature abstractions switch value'''
    return _switches['Abstractions']


def is_unrestricted_s_k_u_enabled():
    '''Returns feature unrestricted_s_k_u switch value'''
    return _switches['UnrestrictedSKU']


def is_example_enabled():
    '''Returns feature example switch value'''
    return _switches['Example']


def is_api_realization_tracking_enabled():
    '''Returns feature api_realization_tracking switch value'''
    return _switches['ApiRealizationTracking']


def is_controller_mgmt_vertical_messaging_u_f_o_enabled():
    '''Returns feature controller_mgmt_vertical_messaging_u_f_o switch value'''
    return _switches['ControllerMgmtVerticalMessagingUFO']


def is_edge_service_in_container_enabled():
    '''Returns feature edge_service_in_container switch value'''
    return _switches['EdgeServiceInContainer']


def is_edge_vertical_messaging_policy_u_f_o_enabled():
    '''Returns feature edge_vertical_messaging_policy_u_f_o switch value'''
    return _switches['EdgeVerticalMessagingPolicyUFO']


def is_edge_ssl_enabled():
    '''Returns feature edge_ssl switch value'''
    return _switches['EdgeSsl']


def is_inventory_container_enabled():
    '''Returns feature inventory_container switch value'''
    return _switches['InventoryContainer']


def is_general_security_settings_profile_enabled():
    '''Returns feature general_security_settings_profile switch value'''
    return _switches['GeneralSecuritySettingsProfile']


def is_l_2_ephemeral_logical_port_synchronous_deletion_enabled():
    '''Returns feature l_2_ephemeral_logical_port_synchronous_deletion switch value'''
    return _switches['L2EphemeralLogicalPortSynchronousDeletion']


def is_vmc_traceflow_enabled_enabled():
    '''Returns feature vmc_traceflow_enabled switch value'''
    return _switches['VmcTraceflowEnabled']


def is_multiple_p_nic_enabled():
    '''Returns feature multiple_p_nic switch value'''
    return _switches['MultiplePNic']


def is_unified_profile_management_support_n_s_group_enabled():
    '''Returns feature unified_profile_management_support_n_s_group switch value'''
    return _switches['UnifiedProfileManagementSupportNSGroup']


def is_p_b_r_enabled():
    '''Returns feature p_b_r switch value'''
    return _switches['PBR']


def is_federation_config_onboarding_enabled():
    '''Returns feature federation_config_onboarding switch value'''
    return _switches['FederationConfigOnboarding']


def is_mpls_enabled():
    '''Returns feature mpls switch value'''
    return _switches['Mpls']


def is_nsx_v_policy_enabled():
    '''Returns feature nsx_v_policy switch value'''
    return _switches['NsxVPolicy']


def is_realized_binding_arp_suppression_enabled():
    '''Returns feature realized_binding_arp_suppression switch value'''
    return _switches['RealizedBindingArpSuppression']


def is_bms_vertical_messaging_u_f_o_enabled():
    '''Returns feature bms_vertical_messaging_u_f_o switch value'''
    return _switches['BmsVerticalMessagingUFO']


def is_intrusion_services_enabled():
    '''Returns feature intrusion_services switch value'''
    return _switches['IntrusionServices']


def is_proximity_routing_enabled():
    '''Returns feature proximity_routing switch value'''
    return _switches['ProximityRouting']


def is_ip_discovery_enabled():
    '''Returns feature ip_discovery switch value'''
    return _switches['IpDiscovery']


def is_nsxt_on_vds_enabled():
    '''Returns feature nsxt_on_vds switch value'''
    return _switches['NsxtOnVds']


def is_network_encryption_enabled():
    '''Returns feature network_encryption switch value'''
    return _switches['NetworkEncryption']


def is_nsxt_on_vds_stateless_esx_enabled():
    '''Returns feature nsxt_on_vds_stateless_esx switch value'''
    return _switches['NsxtOnVdsStatelessEsx']


def is_directory_service_sync_nodes_enabled():
    '''Returns feature directory_service_sync_nodes switch value'''
    return _switches['DirectoryServiceSyncNodes']


def is_network_encryption_override_enabled():
    '''Returns feature network_encryption_override switch value'''
    return _switches['NetworkEncryptionOverride']


def is_l_s_realization_via_c_c_p_enabled():
    '''Returns feature l_s_realization_via_c_c_p switch value'''
    return _switches['LSRealizationViaCCP']


def is_ospf_enabled():
    '''Returns feature ospf switch value'''
    return _switches['Ospf']


def is_mp_clustering_enabled():
    '''Returns feature mp_clustering switch value'''
    return _switches['MpClustering']


def is_virtual_server_resource_limits_enabled():
    '''Returns feature virtual_server_resource_limits switch value'''
    return _switches['VirtualServerResourceLimits']


def is_nsx_dashboard_widget_config_enabled():
    '''Returns feature nsx_dashboard_widget_config switch value'''
    return _switches['NsxDashboardWidgetConfig']


def is_pace_cluster_node_deployment_enabled():
    '''Returns feature pace_cluster_node_deployment switch value'''
    return _switches['PaceClusterNodeDeployment']


def is_identity_firewall_enabled():
    '''Returns feature identity_firewall switch value'''
    return _switches['IdentityFirewall']


def is_u_r_l_filtering_enabled():
    '''Returns feature u_r_l_filtering switch value'''
    return _switches['URLFiltering']


def is_t_n_upgrade_to_vds_enabled():
    '''Returns feature t_n_upgrade_to_vds switch value'''
    return _switches['TNUpgradeToVds']


def is_nsx_uens_enabled():
    '''Returns feature nsx_uens switch value'''
    return _switches['NsxUens']


def is_service_config_enabled():
    '''Returns feature service_config switch value'''
    return _switches['ServiceConfig']


def is_auto_tn_enabled():
    '''Returns feature auto_tn switch value'''
    return _switches['AutoTn']


def is_inventory_applications_enabled():
    '''Returns feature inventory_applications switch value'''
    return _switches['InventoryApplications']


def is_pcg_unified_messaging_enabled():
    '''Returns feature pcg_unified_messaging switch value'''
    return _switches['PcgUnifiedMessaging']


def is_s_r_m_cold_migration_enabled():
    '''Returns feature s_r_m_cold_migration switch value'''
    return _switches['SRMColdMigration']


def is_unified_messaging_enabled():
    '''Returns feature unified_messaging switch value'''
    return _switches['UnifiedMessaging']


def is_inventory_edge_install_enabled():
    '''Returns feature inventory_edge_install switch value'''
    return _switches['InventoryEdgeInstall']


def is_policy_firewall_identity_config_enabled():
    '''Returns feature policy_firewall_identity_config switch value'''
    return _switches['PolicyFirewallIdentityConfig']


def is_lambda_querying_enabled():
    '''Returns feature lambda_querying switch value'''
    return _switches['LambdaQuerying']


def is_custom_role_enabled():
    '''Returns feature custom_role switch value'''
    return _switches['CustomRole']


def is_ls_realization_via_v_c_enabled():
    '''Returns feature ls_realization_via_v_c switch value'''
    return _switches['LsRealizationViaVC']


def is_destination_filtering_enabled():
    '''Returns feature destination_filtering switch value'''
    return _switches['DestinationFiltering']


def is_o_i_d_c_enabled():
    '''Returns feature o_i_d_c switch value'''
    return _switches['OIDC']


def get_switches():
    return _switches