#!/bin/sh

# This script should only be used as directed by the documentation for "restore".

# User will login to the root on ESX/KVM or Edge node and run this script
# This script is used to reset the NSX config from the node.
# When the script is called with 'transport_node' option,
#   it will only delete the logical config and stop serving the network.
# If the node is registered to the NSX Manager as a fabric node,
#   it continues to be that way.
# When the script is called with 'fabric_node' option,
#   it will delete the logical config, stop serving the network and
#   also stop attempting communicating to NSX Manager and NSX Central Control Plane.

EDGE_NSXA_SERVICE_NAME=nsx-edge-nsxa
NSXA_MPA_DB=/config/vmware/edge/nsxa-mpa.pb
NSXA_ID_DB=/var/run/vmware/edge/*.saved
NSXPROXY_DB=/etc/vmware/nsx/controller-info.xml

log() {
  message=$1
  SCRIPTNAME=`basename "$0"`
  echo "$SCRIPTNAME: $message"
}

kvm_tn_cleanup() {
    service nsx-agent stop
    # remove managed bridge
    mbr=$(ovs-vsctl find br external_ids:bridge-id=nsx-managed | grep name | awk '{print $3}')
    [ ! -z $mbr ] && ovs-vsctl del-br $mbr
    # remove transport bridges
    mbx=$(ovs-vsctl find br | grep nsx-switch | awk '{print $3}' | tr -d '"')
    [ ! -z $mbx ] && ovs-vsctl del-br $mbx
    service nsx-agent start
}

bms_app_intf_cleanup() {
    # remove attached application interface
    /opt/vmware/nsx-opsagent/scripts/app_vif_install.sh -c restore
}

bms_stale_config_cleanup() {
    # remove stale config
    case $nodetype in
      nsx-baremetal-ubuntu-server)
      sed -i "/^auto nsx-vtep0.0/,/^up ifconfig nsx-vtep0.0 up$/d" /etc/network/interfaces
      ;;
    esac
}

reset_nsxproxy() {
    node_type=$1
    if [ -e $NSXPROXY_DB ]; then
        log "INFO: Resetting nsx-proxy"
        echo > $NSXPROXY_DB
        case $node_type in
          nsx-edge)
            su admin -c 'set debug-mode; set nsx-proxy controllers file /etc/vmware/nsx/controller-info.xml'
            ;;
          nsx-sleskvm|nsx-baremetal-suse-server|nsx-centoskvm|nsx-baremetal-centos-server|nsx-baremetal-centos-container|nsx-esx|nsx-rhelkvm|nsx-baremetal-rhel-server|nsx-baremetal-rhel-container|nsx-ubuntukvm|nsx-baremetal-ubuntu-server|nsx-baremetal-oel-server)
            /opt/vmware/nsx-cli/bin/scripts/nsxcli -c \
                'set debug-mode; set nsx-proxy controllers file /etc/vmware/nsx/controller-info.xml'
            ;;
          *)
            log "reset_nsxproxy: Invalid node-type $node_type found"
            ;;
        esac
        if [ $? -ne 0 ]; then
            log "ERROR: Failed to reset $NSXPROXY_SERVICE_NAME"
        fi
    fi
}

esx_nsx_vib_cleanup() {
    rm /tmp/nsx-lcp-*.zip
    vib_details=$(localcli software vib list | grep ^nsx | sed 's/  */ /g' | cut -d' ' -f1 | awk '$1=$1' ORS=' -n ')
    vib_details=${vib_details%???}
    vsipfwlib_vib='vsipfwlib'
    log "INFO: Command to uninstall NSX : localcli software vib remove -n $vib_details -n $vsipfwlib_vib"
    localcli software vib remove -n $vib_details -n $vsipfwlib_vib
    status=$?
    if [ $status != 0 ]; then
        error_exit "Failed to remove NSX" $status
    fi
}

kvm_ubuntu_nsx_dpkg_cleanup() {
    rm -f /tmp/nsx-lcp-*.tar.gz
    rm -rf /tmp/nsx-lcp-xenial*
    non_nsx_packages="openvswitch-switch openvswitch-datapath-dkms openvswitch-pki python-openvswitch openvswitch-common libopenvswitch"
    # nsx-cli and nsx-python-logging packages have to be removed after all
    # the rest of the packages because dpkg is dumb and will purge the
    # files that nsx-cli is using (i.e. this script).  This only happens
    # in the nsxcli -c form of the del nsx command.
    # See bz 2598885
    nsx_packages=$(apt list --installed | grep nsx | awk -F '/' '{print $1}' | sed -e "s/nsx-cli//" | sed -e "s/nsx-python-logging//" | tr '\n' ' ')
    log "INFO: Command to uninstall NSX : dpkg --purge $nsx_packages $non_nsx_packages"
    dpkg --purge $nsx_packages $non_nsx_packages
    status=$?
    if [ $status != 0 ]; then
        error_exit "Failed to remove NSX" $status
    fi
    dpkg --purge nsx-python-logging nsx-cli
    status=$?
    if [ $status != 0 ]; then
        error_exit "Failed to remove NSX" $status
    fi
}

kvm_rhel_nsx_rpm_cleanup() {
    rm /tmp/nsx-lcp-*.tar.gz
    rm -r /tmp/nsx-lcp-rhel*
    non_nsx_packages="openvswitch-kmod openvswitch openvswitch-selinux-policy"
    nsx_packages=$(rpm -qa | grep ^nsx | tr '\n' ' ')
    log "INFO: Command to uninstall NSX : rpm -e $nsx_packages $non_nsx_packages"
    rpm -e $nsx_packages $non_nsx_packages
    status=$?
    if [ $status != 0 ]; then
        error_exit "Failed to remove NSX" $status
    fi
}

kvm_sles_nsx_rpm_cleanup() {
    rm /tmp/nsx-lcp-*.tar.gz
    rm -r /tmp/nsx-lcp-sles*
    non_nsx_packages=$(rpm -qa | grep ^openvswitch | tr '\n' ' ')
    nsx_packages=$(rpm -qa | grep ^nsx | tr '\n' ' ')
    log "INFO: Command to uninstall NSX : rpm -e $nsx_packages $non_nsx_packages"
    rpm -e $nsx_packages $non_nsx_packages
    status=$?
    if [ $status != 0 ]; then
        error_exit "Failed to remove NSX" $status
    fi
}

error_exit() {
    echo "$1" >&2   ## Send message to stderr.
    log "$1"
    exit "${2:-1}"  ## Return a code specified by $2 or 1 by default.
}

nodetype=$2
option=$1

if [ "$option" = "fabric_node" ] || [ "$option" = "transport_node" ]; then
    log "INFO: NSX reset script called with argument $option on $nodetype"
else
    log "Usage : <transport_node | fabric_node>"
    return
fi

case $nodetype in
  nsx-edge)
    log "INFO: Run transport_node reset on Edge node"
    log "INFO: Resetting $EDGE_NSXA_SERVICE_NAME"
    service $EDGE_NSXA_SERVICE_NAME stop
    rm -f $NSXA_MPA_DB $NSXA_ID_DB
    service $EDGE_NSXA_SERVICE_NAME start
    ;;
  nsx-esx)
    log "INFO: Run transport_node reset on ESX node"
    /opt/vmware/nsx-cli/bin/scripts/nsxcli -c \
      'set debug-mode; set service nsx-opsagent nsxa reset'
    status=$?
    if [ $status -ne 0 ]; then
        error_exit "ERROR: Failed to reset nsxa app of nsx-opsagent. Please check ospagent logs for more details." $status
    fi
    /etc/init.d/nsx-proxy restart
    ;;
  nsx-sleskvm|nsx-centoskvm|nsx-baremetal-centos-container|nsx-esx|nsx-rhelkvm|nsx-baremetal-rhel-container|nsx-ubuntukvm)
    log "INFO: Run transport_node reset on KVM node"
    kvm_tn_cleanup
    ;;
  nsx-baremetal-suse-server|nsx-baremetal-centos-server|nsx-baremetal-rhel-server|nsx-baremetal-ubuntu-server|nsx-baremetal-oel-server)
    log "INFO: Run transport_node reset on BMS node"
    bms_app_intf_cleanup
    kvm_tn_cleanup
    bms_stale_config_cleanup
    ;;
  *)
    log "INFO: Invalid node-type $nodetype found"
    ;;
esac

if [ "$option" = "fabric_node" ]; then
  case $nodetype in
    nsx-edge)
        log "INFO: Run fabric_node reset on Edge node"
        /opt/vmware/nsx-mpa/mpaconfigrestore.sh
        reset_nsxproxy $nodetype
        service $EDGE_NSXA_SERVICE_NAME restart
        service nsx-edge-datapath restart
        ;;
    nsx-centoskvm|nsx-baremetal-centos-server|nsx-baremetal-centos-container)
        log "INFO: Run fabric_node reset on CENTOSKVM node"
        reset_nsxproxy $nodetype
        kvm_rhel_nsx_rpm_cleanup
        ;;
    nsx-sleskvm|nsx-baremetal-suse-server)
        log "INFO: Run fabric_node reset on SLESKVM node"
        reset_nsxproxy $nodetype
        kvm_sles_nsx_rpm_cleanup
        ;;
    nsx-esx)
        log "INFO: Run fabric_node reset on ESX node"
        reset_nsxproxy $nodetype
        esx_nsx_vib_cleanup
        ;;
    nsx-rhelkvm|nsx-baremetal-rhel-server|nsx-baremetal-rhel-container|nsx-baremetal-oel-server)
        log "INFO: Run fabric_node reset on RHELKVM node"
        reset_nsxproxy $nodetype
        kvm_rhel_nsx_rpm_cleanup
        ;;
    nsx-ubuntukvm|nsx-baremetal-ubuntu-server)
        log "INFO: Run fabric_node reset on UBUNTUKVM node"
        reset_nsxproxy $nodetype
        kvm_ubuntu_nsx_dpkg_cleanup
        ;;
    *)
        log "INFO: Invalid node-type found"
        ;;
  esac
fi;
