# **********************************************************************
# Copyright 2020 VMware, Inc.  All rights reserved. VMware Confidential.
# **********************************************************************
# $Id$
# $DateTime$
# $Change$
# $Author$
# **********************************************************************
import subprocess
import sys
sys.path.append("/opt/vmware/nsx-common/python")
import nsx_logging


lg = nsx_logging.getLogger(__name__)


NSX_CUSTOM_TARGET = "nsx-custom.target"


def get_process_data():
    """Return ps output for parsing service properties."""
    result = []
    cmd = ["ps", "-weo", "pid,ppid,cmd"]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = po.communicate()[0].decode()
    lines = out.split("\n")
    for line in lines:
        line = line.strip()
        if not line:
            continue
        parts = line.split()
        result.append((parts[0], parts[1], " ".join(parts[2:])))
    return result


def valid_listen_address(address):
    """Return True if address is a valid ip_address:port; otherwise, False."""
    # Do address range checks here
    ip, port = address.split(":")
    try:
        port = int(port)
        if port < 1 or port > 65535:
            return False
    except ValueError:
        return False
    try:
        socket.inet_aton(ip)
    except socket.error:
        return False
    return True


def query_service_via_curl(url):
    cmd = ['curl', '-s', '-XGET', url]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = po.communicate()
    return po.returncode, out, err


def query_service_via_nc(port, message):
    cmd = ['nc', 'localhost', port]
    po = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                       stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = po.communicate(input=message)
    return po.returncode, out, err


def get_postgres_db_names():
    pg_cmd = 'psql -c \"SELECT datname FROM pg_database;\"'
    cmd = ['sudo', 'su', '-s', '/bin/bash', '-', 'postgres', '-c', pg_cmd]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = po.communicate()
    return po.returncode, out, err


def update_rc_defaults(init_script_name):
    cmd = ["update-rc.d", init_script_name, "defaults"]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8')  for p in po.communicate()]
    return po.returncode, out, err


def update_rc_remove(init_script_name):
    cmd = ["update-rc.d", "-f", init_script_name, "remove"]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8')  for p in po.communicate()]
    return po.returncode, out, err


def service_action_with_info(init_script_name, action, info):
    if (info is not None):
        lg.info("Service Action: " + action + " " + init_script_name + " - " + info)
    else:
        lg.info("Service Action: " + action + " " + init_script_name)

    if init_script_name == "ssh" and action in ["restart", "start"]:
        # Special case for service ssh restart/start because it is known to
        # sometimes block indefinitely.
        cmd = ["timeout", "--preserve-status", "30", "service", init_script_name, action]
        po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = [p.decode('utf-8')  for p in po.communicate()]
        # returncode=143 if timeout occurs
        # returncode=0 on command success
        if po.returncode:
            # Try again
            po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            out, err = [p.decode('utf-8')  for p in po.communicate()]
    else:
        # Regular case
        cmd = ["service", init_script_name, action]
        po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = [p.decode('utf-8')  for p in po.communicate()]
    return po.returncode, out, err


def systemctl_action(init_script_name, action):
    cmd = ["systemctl", action, init_script_name]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8')  for p in po.communicate()]
    return po.returncode, out, err


def service_action(init_script_name, action):
    return service_action_with_info(init_script_name, action, None)


def service_action(init_script_name, action):
    return service_action_with_info(init_script_name, action, None)


def service_status(init_script_name):
    if init_script_name is None:
        return 1, None, None
    cmd = ["service", init_script_name, "status"]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8') for p in po.communicate()]
    return po.returncode, out, err


def service_status_by_process(process_id):
    if process_id is None:
        return 1, None, None
    cmd = ["pgrep", "-f", process_id]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8')  for p in po.communicate()]
    return po.returncode, out, err


def is_edge_in_maintenance_mode():
    mode = False
    cmd = ["systemctl", "list-units", "-t", "target"]
    po = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = [p.decode('utf-8')  for p in po.communicate()]
    if out and NSX_CUSTOM_TARGET not in out:
        mode = True
    return mode
