# Copyright (C) 2020-2021 VMware, Inc. All rights reserved.
# -- VMware Confidential
import sha.config

import os
import re
import sys
import time
import traceback
import uuid
from threading import Lock, Thread
from sha.contrib.action.abc_base import Action
from sha.contrib.metric.utils import run_command
from sha.config_monitor import ConfigMonitor

from sha.core.channel.provider.wavefront_proxy_client import metric_to_line_data, tracing_span_to_line_data

from sha.constants import DATA, WAVEFRONT_TYPE, WAVEFRONT_TYPE_METRIC, WAVEFRONT_TYPE_TRACING, \
                          EXPORT_TYPE_WF_METRIC, EXPORT_TYPE_WF_TRACING, EXPORT_TYPE_DUMP_TRACE_FILE, \
                          EXPORT_TYPE_TSDB_METRICS, EXPORT_TYPE_TSDB_METRICSBAG, TYPE, \
                          WAVEFRONT_HUB, TRACEDUMP_HUB, TSDB_HUB


class Exportdata(Action):
    _logger = sha.config.get_logger(__name__)
    _convert_pattern = re.compile(r'[^a-zA-Z0-9-_./,]+')

    @classmethod
    def flat_nested_dict(cls, nested_dict):
        flat_dict = {}
        for (key, value) in nested_dict.items():
            if value is None:
                continue
            if type(value) is dict:
                new_dict = Exportdata.flat_nested_dict(value)
                for (new_key, new_value) in new_dict.items():
                   flat_dict['{0}.{1}'.format(key, new_key)] = new_value
            else:
                try:
                    flat_dict[key] = float(value)
                except Exception as e:
                    cls._logger.info('Failed to convert value:{}'.format(e))
                    pass

        return flat_dict

    @classmethod
    def populate_wavefront_metric_data(cls, **kwargs):
        flat_dict = Exportdata.flat_nested_dict(kwargs['data'])
        wavefront_timestamp = int(kwargs.get('timestamp', time.time()))
        # Get source from kwargs, use NSX host_uuid if not set
        wavefront_source = kwargs.get('source', ConfigMonitor.host_id())
        batch_metric_data = []
        for (key, value) in flat_dict.items():
            # Convert unsupported special char in key to "-"
            one_metric_data = metric_to_line_data(
                name=cls._convert_pattern.sub('-', key),
                value=value, timestamp=wavefront_timestamp,
                source=wavefront_source, tags={}, default_source="")
            batch_metric_data.append(one_metric_data)
        return batch_metric_data

    @classmethod
    def populate_wavefront_tracing_span(cls, **kwargs):
        record = kwargs['data']
        batch_data = []
        for span in record:
            parent = None
            if 'parent' in span:
                parent = uuid.UUID(span['parent'])
            one_tracing = tracing_span_to_line_data(
                name = span['name'],
                start_millis=span['timestamp'],
                duration_millis = span['duration'],
                source = span['source'],
                trace_id = uuid.UUID(span['traceId']),
                span_id = uuid.UUID(span['spanId']),
                parents = [parent],
                follows_from = None,
                tags = span['tags'],
                span_logs = None,
                default_source = "defaultSource")
            batch_data.append(one_tracing)
        return batch_data

    @classmethod
    def do(cls, hubs, *args):
        def kernel(**kwargs):
            cls._logger.debug('signaling: {}'.format(kwargs))
            for export in kwargs['exports']:
                if export == EXPORT_TYPE_WF_METRIC:
                    wavefront_data = Exportdata.populate_wavefront_metric_data(**kwargs)
                    cls._logger.debug('Wavefront metric data ready {}'.format(wavefront_data))
                    hubs[WAVEFRONT_HUB].send({DATA: wavefront_data, WAVEFRONT_TYPE: WAVEFRONT_TYPE_METRIC})
                elif export == EXPORT_TYPE_WF_TRACING:
                    wavefront_data = Exportdata.populate_wavefront_tracing_span(**kwargs)
                    cls._logger.debug('Wavefront tracing span ready {}'.format(wavefront_data))
                    hubs[WAVEFRONT_HUB].send({DATA: wavefront_data, WAVEFRONT_TYPE: WAVEFRONT_TYPE_TRACING})
                elif export == EXPORT_TYPE_DUMP_TRACE_FILE:
                    cls._logger.debug('Filedump tracing span ready {}'.format(kwargs['data']))
                    hubs[TRACEDUMP_HUB].send({DATA: kwargs['data']})
                elif export == EXPORT_TYPE_TSDB_METRICS or export == EXPORT_TYPE_TSDB_METRICSBAG:
                    cls._logger.debug('TSDB MetricAgentMsg ready {}'.format(kwargs['data']))
                    hubs[TSDB_HUB].send({DATA: kwargs['data'], TYPE: export})
                else:
                    cls._logger.debug('Unsupported export: {}'.format(export))

        return kernel
