# Copyright (C) 2020 - 2021 VMware, Inc. All rights reserved.
# -- VMware Confidential
import socket
import struct
import traceback
import sha.config

if sha.config.is_linux_platform():
    from vmware.nsx.sha import sha_pm_ts_pb2 as sha_pm
else:
    from vmware.nsx.sha import sha_pm_pb2 as sha_pm
from sha.contrib.monitor.metricsReceiver.internal.client import Client

class ClientStopException(Exception):
    pass

class Parser(object):
    """
    Receive and parse the clients' message
    """
    def __init__(self, sock, client):
        self._logger = sha.config.get_logger(__name__)
        self._conn = sock
        self._client = client

    def _decode_msg(self, data):
        self._logger.debug("MM: data to be parsed {}".format(data))
        msg = sha_pm.ShaClientMsg()
        msg.ParseFromString(data)
        ns = msg.name_space
        self._logger.debug("MM:name space is {}".format(ns))
        data = []
        for item in msg.item:
            self._logger.debug("MM:data key is {} {}".format(item.key, item.type))
            if item.type == sha_pm.ShaClientItemMsg.ItemMsgType.Value('NUMERICAL'):
                data.append({'key': item.key, 'val': item.numerical_value, 'ts': item.timestamp})
            elif item.type == sha_pm.ShaClientItemMsg.ItemMsgType.Value('BOOLEAN'):
                data.append({'key': item.key, 'val': item.boolean_value, 'ts': item.timestamp})
            elif item.type == sha_pm.ShaClientItemMsg.ItemMsgType.Value('STRING'):
                data.append({'key': item.key, 'val': item.string_value, 'ts': item.timestamp})
            elif (sha.config.is_ua() or sha.config.is_edge()) and \
                 item.type == sha_pm.ShaClientItemMsg.ItemMsgType.Value('TSDB'):
                self._logger.debug("MM:tsdb value is {}".format(item.tsdb_value.SerializeToString()))
                data.append({'key': item.key, 'val': item.tsdb_value, 'ts': item.timestamp})
            else:
                data.append({'key': item.key, 'ts': item.timestamp})
        self._logger.debug("MM:parsed data list is {}".format(data))

        return ns, data

    def _handle_data(self, ns, data):
        for item in data:
            if 'start' in item['key']:
                self._logger.info("MM:Received start command")
                Client.add_client(self._client, ns)
            elif 'stop' in item['key']:
                self._logger.info("MM:Received stop command")
                Client.del_client(ns)
                raise ClientStopException
            elif 'heartbeat' in item['key']:
                self._client.update_heartbeat_ts(item['ts'])
            else:
                self._client.refresh_data(item)

    def recv(self):
        """Anything wrong with reception would cause the reconnection."""
        delimiter = self._conn.recv(9)
        self._logger.debug("MM:delimiter is {}".format(delimiter.decode("utf-8")))
        self._conn.send("bumping".encode("utf-8"))
        if 'SHACLIENT' not in delimiter.decode("utf-8"):
            return
        (length,) = struct.unpack('>I', self._conn.recv(4))
        self._logger.debug("MM:payload length is {}".format(length))
        if length:
            bytes = self._recv(length)
        else:
            raise Exception
        ns, data = self._decode_msg(bytes)
        self._handle_data(ns, data)

    def _recv(self, total_len):
        rcv_len = 0
        res = None
        READ_LEN = 1000

        if READ_LEN > total_len:
            cur = self._conn.recv(total_len)
            return cur

        while rcv_len < total_len:
            if total_len - rcv_len > READ_LEN:
                cur = self._conn.recv(READ_LEN)
            else:
                cur = self._conn.recv(total_len - rcv_len)
            if res is None:
                res = cur
            else:
                res += cur
            rcv_len = len(res)

        return res

