# Copyright (C) 2019 VMware, Inc. All rights reserved.
# -- VMware Confidential

import ctypes
import threading
import time
import traceback

import sha.config
from sha.core.channel.provider.base import BaseProvider
from vmware.nsx.sha import sha_pb2

if sha.config.is_ua() or sha.config.is_gm():
    CONFIG_SYNC_INTERVAL = 60
    CONFIG_SYNC_RETRY = 20
    config_synced = False


class MpaProvider(BaseProvider):
    """The MPA Provider"""

    def __init__(self):
        self._name = "MpaProvider"
        self._logger = sha.config.get_logger(__name__)
        self._logger.info("initiating channel provider <{}>".format(self.__class__.name))

        self._logger.info("platform is {}".format(sha.config.SYSTEM))
        if sha.config.SYSTEM == sha.config.PLATFORM_WINDOWS:
            self.sha_client = ctypes.CDLL('sha_client.dll')
        else:
            self.sha_client = ctypes.CDLL('libshaclientshared.so.0')
        if sha.config.is_ua() or sha.config.is_gm():
            self.sha_client.set_callback.argtypes = [ctypes.py_object]
            self.sha_client.set_callback(handle)
        self._mutex = threading.Lock()
        self.sha_client_thread = threading.Thread(target=self.sha_client.createNSXShaClient, daemon=True,
                                                  name="nsx-sha-create-nsx-sha-client")
        self.sha_client_thread.start()
        # wait for sha client thread finish creation
        time.sleep(5)

    def config_sync(self):
        if not sha.config.is_ua() and not sha.config.is_gm():
            return

        # Construct config request message
        from vmware.nsx.sha import sha_pb2
        from sha.config_monitor import ConfigMonitor
        report_msg = sha_pb2.ShaReportMsg()
        report_msg.oid.left, report_msg.oid.right = ConfigMonitor.host_id_low_high()
        status_msg = report_msg.status.add()
        status_msg.type = sha_pb2.ShaStatusMsg.ShaStatusType.Value('CONFIG_REQUEST')
        status_msg.config_request.SetInParent()
        pb_msg = report_msg.SerializeToString()
        msg = ReportMsg(b'', pb_msg, b'com.vmware.nsx.sha.ShaReportMsg')

        global config_synced
        retry_time = 0
        while not config_synced and retry_time < CONFIG_SYNC_RETRY:
            self.sync(msg)
            retry_time += 1
            self._logger.info("The {}th try to send config request to MP".format(retry_time))
            time.sleep(CONFIG_SYNC_INTERVAL)

        if not config_synced:
            self._logger.warning("{} failed in config full sync after {} retries."
                                 .format(self.name, retry_time))

    def sync(self, msg):
        with self._mutex:
            payload = msg.payload
            payload_len = len(payload)
            payload_type = msg.payload_type
            topic = msg.topic
            try:
                self._logger.info("before sendNSXShaMessage, msg:{}, payload_len:{}".format(msg, payload_len))
                self.sha_client.sendNSXShaMessage(payload, payload_len, payload_type, topic)
                self._logger.info("sendNSXShaMessage successfully.")
            except Exception as e:
                trace = traceback.format_exc()
                self._logger.warning("{} failed in sendNSXShaMessage via SHA client, msg:{}, Exception:{}, trace:{}"
                                     .format(self.name, msg, e, trace))

    def close(self):
        self._logger.info("{} is destroying SHA client.".format(self.name))
        with self._mutex:
            try:
                self.sha_client.destroyNSXShaClient()
                del self.sha_client
                self._logger.info("{} finished destroying SHA client.".format(self.name))
            except Exception:
                trace = traceback.format_exc()
                self._logger.warning("{} failed in destroy SHA client, trace:{}".format(self.name, trace))

    @property
    def name(self):
        return self._name


class ReportMsg:
    def __init__(self, topic, payload, payload_type):
        self._topic = topic
        self._payload = payload
        self._payload_type = payload_type

    def __repr__(self):
        return 'ReportMsg: topic:{}, payload:{}, payload_type:{}' \
            .format(self.topic, sha_pb2.ShaReportMsg().FromString(self.payload), self.payload_type)

    @property
    def topic(self):
        return self._topic

    @property
    def payload(self):
        return self._payload

    @property
    def payload_type(self):
        return self._payload_type


mpa_logger = sha.config.get_logger("MpaProvider")


def handle(msg, reply_to, cor_id):
    mpa_logger.info("MpaProvider callback is called")
    if sha.config.is_ua() or sha.config.is_gm():
        from sha.core.profile._global_config_resolver import TraceResolver, CollectorResolver, TSDBResolver
        from sha.core.profile._host_config_resolver import HostConfigResolver
        from vmware.nsx.sha import sha_pb2
        from google.protobuf import text_format
        def get_config_type(name):
            return sha_pb2.ConfigEntryMsg.ConfigType.DESCRIPTOR.values_by_name[name].number

        def get_action_type(name):
            return sha_pb2.ConfigEntryMsg.ActionType.DESCRIPTOR.values_by_name[name].number

        global config_synced
        if not config_synced:
            config_synced = True
            mpa_logger.info("MpaProvider config synced flag changed to True.")
        try:
            sha_config_msg = text_format.Parse(msg, sha_pb2.ShaConfigMsg())
            mpa_logger.info("ua_config reply to {} , cor_id {}".format(
                            reply_to, cor_id))

            for entry in sha_config_msg.config:
                if entry.config_type == get_config_type('TRACE'):
                    if entry.action == get_action_type('UPDATE'):
                        TraceResolver.set(entry.trace_config)
                    elif entry.action == get_action_type('CLEAR'):
                        TraceResolver.clear()
                    else:
                        mpa_logger.warning("Msg with invalid action: {}".format(entry))
                elif entry.config_type == get_config_type('COLLECTOR'):
                    if entry.action == get_action_type('UPDATE'):
                        CollectorResolver.set(entry.collector_config)
                    elif entry.action == get_action_type('CLEAR'):
                        CollectorResolver.clear()
                    else:
                        mpa_logger.warning("Msg with invalid action: {}".format(entry))
                elif entry.config_type == get_config_type('TSDB'):
                    # TSDB config
                    if entry.action == get_action_type('UPDATE'):
                        # Update certificate
                        HostConfigResolver.set(entry.tsdb_config)
                        # Update enable flag
                        TSDBResolver.set(entry.tsdb_config)
                    elif entry.action == get_action_type('CLEAR'):
                        # clear enable flag
                        TSDBResolver.clear()
                        # Clear certificate
                        HostConfigResolver.clear()
                    else:
                        mpa_logger.warning("Msg with invalid action: {}".format(entry))
                else:
                    mpa_logger.warning("Msg with invalid type: {}".format(entry))
        except Exception as e:
            trace = traceback.format_exc()
            mpa_logger.warning("{} failed in handle message from SHA Client, msg:{}, Exception:{}, trace:{}"
                               .format("ua_config", msg, e, trace))


