# Copyright (C) 2019-2021 VMware, Inc. All rights reserved.
# -- VMware Confidential
import json
import os
import traceback
import yaml
from sha.core.profile.profile_collection import ProfileCollection
from sha.core.plugin.custom_plugin_cache import CustomPluginCache
import sha.config
from sha.constants import CHECK_INTERVAL, REPORT_INTERVAL, \
    SMALLEST_REPORT_INTERVAL_IF_CHANGE, ENABLE, PROFILE_NAME

from sha.utils import convert_to_uuid_str


class Resolver:
    _logger = sha.config.get_logger(__name__)
    # store profile_collection for each plugin
    # plugin_id -> profile_collection
    _profiles = dict()

    DEFAULT_PROFILE_NAME = 'default-profile'

    HOST_METRICS_MONITOR_ID = 'a4e2b84b-0bed-468e-a6e8-f3f7050f02c8'
    EDGE_STATUS_MONITOR_ID = 'cfddbc5f-0d15-4c2a-bde9-f62de21007c9'
    EDGE_FW_STATS_MONITOR_ID = '26d87226-1673-4c3c-9b56-50d85fc57479'
    EDGE_FW_CONN_MONITOR_ID = '63b58bc1-6c61-4414-92b5-19ef57e84088'
    EDGE_IPSEC_VPN_STATS_MONITOR_ID = '15b20d7b-173d-4c60-a181-48a54b76d627'
    EDGE_MGMT_INTERFACE_STATS_MONITOR_ID = 'b307b1dc-ed4b-521a-85e3-552231cdb7cc'
    EDGE_DP_THREAD_STATUS_MONITOR_ID = '15af9bfb-8559-5e04-847d-4398d7b4d883'
    VDR_STATS_MONITOR_ID = 'e819d635-472f-4a58-80cd-55639f2edc2f'
    # TODO add uuid for each plugin at MP side
    HYPERBUS_MONITOR_ID = '6dd5dfe8-f545-4e9b-94cc-30fa320797ff'
    NODE_AGENT_MONITOR_ID = '3bea9fe0-d7d9-4ec4-8600-750e1366fb33'
    DAEMON_HEALTH_MONITOR_ID = '217c1efc-c0b1-41ca-a65a-e783f7303b6f'
    UPLINK_STATUS_MONITOR_ID = 'cd8e3761-7634-42d5-ab2c-aec72cab31cb'
    MIRRORSTACK_MONITOR_ID = '60cd6963-cdb6-44b2-aa4d-54f34984a655'
    TOP_PROCESS_MONITOR_ID = '9829c931-8cd4-4f8f-b250-2f5b36401bae'
    AUDIT_LOG_HEALTH_FOR_LOCAL_MONITOR_ID = 'c35fe92c-2510-4bab-a337-3cfcfbd5333b'
    AUDIT_LOG_HEALTH_FOR_REMOTE_MONITOR_ID = '8f03df3d-2ffd-49e4-9232-64901440c33d'
    MANAGER_FQDN_LOOKUP_FAILURE_MONITOR_ID = '8aa757a6-4d15-4325-9779-2cc4fc79726f'
    MANAGER_FQDN_REVERSE_LOOKUP_FAILURE_MONITOR_ID = '747e7b82-e09c-462a-aa52-892df0b9654e'
    NODE_LATENCY_MONITOR_ID = 'e07817bc-9f5f-4bed-9ac7-6ecf2f9f165d'
    EDGE_L3_MONITOR_ID = 'a3dad146-e277-4617-9c12-d5b4b85f36c6'
    EDGE_APPLIANCE_MONITOR_ID = '56936d6a-f0f7-4bf0-9934-d670602b5310'
    PROCESS_STATUS_MONITOR_ID = '97821e38-ae9a-11ea-a244-ef6bd3773af9'
    SERVICE_CFGAGENT_MONITOR_ID = 'f46b5286-c574-11ea-8d97-0050569fb78e'
    SERVICE_OPSAGENT_MONITOR_ID = '22ec835a-c575-11ea-8d97-0050569fb78e'
    TRACE_MONITOR_ID = '97821e38-ae9a-11ea-a244-ef6bd3773af8'
    SYS_STATS_MONITOR_ID = '6837ba7a-39cd-11eb-9f83-020068594730'
    EDGE_DP_STATS_MONITOR_ID = 'ded8428a-c887-479f-a3a4-4a9f7165d80c'
    TSDB_MONITOR_ID = '93b63dde-40f3-11eb-91f7-020062d454e5'
    NESTDB_REMEDY_ID = '08878948-f2ae-42b6-8c63-c03091cac158'
    EDGE_LR_PORT_STATS_MONITOR_ID = 'a8d9748a-d02d-424d-ac89-ff035e22cc5d'
    TLS_STATS_MONITOR_ID = 'cdb2ccb2-d75c-5bb4-8fc4-bddf6033d6cc'
    LB_STATUS_MONITOR_ID = 'df3169ea-8d43-4968-87cb-0ac138fb5747'
    IDPS_STATS_MONITOR_ID = '16d2490a-e505-5f25-8259-cddefbf8040b'
    WAVEFRONT_MONITOR_ID = '53c8736c-871e-4277-8a48-a0cca656f196'

    _predefined_plugin_id_name_mapper = {
        HOST_METRICS_MONITOR_ID: 'host_metrics_monitor',
        EDGE_STATUS_MONITOR_ID: 'edge_status_monitor',
        EDGE_FW_STATS_MONITOR_ID: 'edge_fw_stats_monitor',
        EDGE_FW_CONN_MONITOR_ID: 'edge_fw_conn_monitor',
        EDGE_IPSEC_VPN_STATS_MONITOR_ID: 'edge_ipsec_vpn_stats_monitor',
        VDR_STATS_MONITOR_ID: 'vdr_stats_monitor',
        HYPERBUS_MONITOR_ID: 'hyperbus_monitor',
        NODE_AGENT_MONITOR_ID: 'node_agent_monitor',
        DAEMON_HEALTH_MONITOR_ID: 'daemon_health_monitor',
        UPLINK_STATUS_MONITOR_ID: 'uplink_status_monitor',
        MIRRORSTACK_MONITOR_ID: 'mirrorstack_monitor',
        TOP_PROCESS_MONITOR_ID: 'top_process_monitor',
        AUDIT_LOG_HEALTH_FOR_LOCAL_MONITOR_ID: 'audit_log_health_for_local_monitor',
        AUDIT_LOG_HEALTH_FOR_REMOTE_MONITOR_ID: 'audit_log_health_for_remote_monitor',
        MANAGER_FQDN_LOOKUP_FAILURE_MONITOR_ID: 'manager_fqdn_lookup_failure_monitor',
        MANAGER_FQDN_REVERSE_LOOKUP_FAILURE_MONITOR_ID: 'manager_fqdn_reverse_lookup_failure_monitor',
        NODE_LATENCY_MONITOR_ID: 'ua_node_latency_monitor',
        EDGE_L3_MONITOR_ID: 'edge_l3_monitor',
        EDGE_APPLIANCE_MONITOR_ID: 'edge_appliance_monitor',
        PROCESS_STATUS_MONITOR_ID: 'process_status_monitor',
        EDGE_MGMT_INTERFACE_STATS_MONITOR_ID: 'edge_mgmt_interface_monitor',
        EDGE_DP_THREAD_STATUS_MONITOR_ID: 'rcu_event_monitor',
        SERVICE_CFGAGENT_MONITOR_ID: 'service_cfgagent_monitor',
        SERVICE_OPSAGENT_MONITOR_ID: 'service_opsagent_monitor',
        TRACE_MONITOR_ID: 'trace_monitor',
        SYS_STATS_MONITOR_ID: 'sys_stats_monitor',
        EDGE_DP_STATS_MONITOR_ID: 'edge_dp_stats_monitor',
        TSDB_MONITOR_ID: 'tsdb_monitor',
        NESTDB_REMEDY_ID: 'nestdb_remedy',
        EDGE_LR_PORT_STATS_MONITOR_ID: 'edge_lr_port_stats_monitor',
        TLS_STATS_MONITOR_ID: 'tls_stats_monitor',
        LB_STATUS_MONITOR_ID: 'lb_status',
        IDPS_STATS_MONITOR_ID: 'idps_stats_monitor',
        WAVEFRONT_MONITOR_ID: 'wavefront_monitor',
    }

    _plugins_not_check_by_interval = {UPLINK_STATUS_MONITOR_ID, PROCESS_STATUS_MONITOR_ID,
                                      SERVICE_CFGAGENT_MONITOR_ID, SERVICE_OPSAGENT_MONITOR_ID,
                                      TRACE_MONITOR_ID, TSDB_MONITOR_ID, WAVEFRONT_MONITOR_ID}

    # Add plugin id to _plugins_report_to_mp if plugin profile should contain fields
    # 'REPORT_INTERVAL' & 'SMALLEST_REPORT_INTERVAL_IF_CHANGE' to restrict report frequency to MP
    _plugins_report_to_mp = \
        {DAEMON_HEALTH_MONITOR_ID, HYPERBUS_MONITOR_ID, NODE_AGENT_MONITOR_ID, MIRRORSTACK_MONITOR_ID}

    if sha.config.is_ua() or sha.config.is_gm():
        _type_id_mapper = {}
    else:
        # for upgrade case
        # from get profile by name (Grindcore) to get profile by id (Grindcore-main & Highline)
        from vmware.nsx.nestdb import nestdb_schema_upm_pb2 as schema_pb2
        _type_id_mapper = {
            schema_pb2.SystemHealthProfileMsg.SystemHealthType.Value(
                'SYSTEM_HEALTH_TYPE_HYPERBUS'): HYPERBUS_MONITOR_ID,
            schema_pb2.SystemHealthProfileMsg.SystemHealthType.Value(
                'SYSTEM_HEALTH_TYPE_NODE_AGENT'): NODE_AGENT_MONITOR_ID,
            schema_pb2.SystemHealthProfileMsg.SystemHealthType.Value(
                'SYSTEM_HEALTH_TYPE_TN_AGENT'): DAEMON_HEALTH_MONITOR_ID,
            schema_pb2.SystemHealthProfileMsg.SystemHealthType.Value(
                'SYSTEM_HEALTH_TYPE_UPLINK'): UPLINK_STATUS_MONITOR_ID,
        }

    @classmethod
    def get(cls, plugin_id):
        """
        get the profile of a specific plugin, return the default one if no profile is dispatched from ccp.
        :rtype: dict
        """
        if plugin_id in cls._profiles:
            profile = cls._profiles[plugin_id].get()
            if profile is not None:
                return profile

        # predefined
        if cls.is_predefined(plugin_id):
            default_path = \
                os.path.join(sha.config.SHA_ROOT_DIR, 'plugins', cls.get_predefined_plugin_name(plugin_id) + '.yml')
            with open(default_path, 'r') as f:
                default_profile = yaml.load(f.read(), Loader=yaml.Loader)
        else:
            # custom
            plugin = CustomPluginCache.get_plugin(plugin_id)
            default_profile = yaml.load(plugin.config, Loader=yaml.Loader)
        default_profile[PROFILE_NAME] = cls.DEFAULT_PROFILE_NAME
        return default_profile

    @classmethod
    def is_predefined(cls, plugin_id):
        return plugin_id in cls._predefined_plugin_id_name_mapper

    @classmethod
    def get_predefined_plugin_name(cls, plugin_id):
        cls._logger.debug("got plugin name:{} for plugin id:{}"
                          .format(cls._predefined_plugin_id_name_mapper[plugin_id], plugin_id))
        return cls._predefined_plugin_id_name_mapper[plugin_id]

    @classmethod
    def is_positive_int(cls, to_check):
        """
        Check whether the input param 'to_check' is an integer and larger than 0
        :return: True - Positive int; False - Not Positive int
        """
        if to_check is None:
            return False
        if not isinstance(to_check, int):
            return False
        return to_check > 0

    @classmethod
    def is_valid_profile(cls, plugin_id, sha_profile_pb2):
        """
        Validate profile configuration.
        All profiles should contain field 'enabled'.
        For those plugins checked by interval, its corresponding profile should also contain valid interval value.
        For those plugins report to MP, its corresponding profile should also contain valid REPORT_INTERVAL
        and SMALLEST_REPORT_INTERVAL_IF_CHANGE.
        :param plugin_id:
        :param sha_profile_pb2: profile msg from nestdb
        :return: True - Valid, False - Invalid
        """
        if not plugin_id:
            cls._logger.warn('Not a valid profile as plugin_id is empty, profile:{}, plugin_id:{}'
                             .format(str(sha_profile_pb2), plugin_id))
            return False

        plugin_name = cls._predefined_plugin_id_name_mapper.get(plugin_id)

        if not sha_profile_pb2.HasField('enabled'):
            cls._logger.warn('Not a valid profile as enabled is not set, profile:{}, plugin_name:{}, plugin_id:{}'
                             .format(str(sha_profile_pb2), plugin_name, plugin_id))
            return False

        try:
            content = json.loads(sha_profile_pb2.content)
        except Exception as e:
            trace = traceback.format_exc()
            cls._logger.warn('Not a valid profile as error in parsing content, trace:{}, profile:{}, plugin_name:{}, '
                             'plugin_id:{}'.format(trace, str(sha_profile_pb2), plugin_name, plugin_id))
            return False

        if plugin_id in cls._plugins_not_check_by_interval:
            cls._logger.debug('Valid profile:{}, plugin_name:{}, plugin_id:{}'
                              .format(str(sha_profile_pb2), plugin_name, plugin_id))
            return True

        if CHECK_INTERVAL not in content \
                or not cls.is_positive_int(content[CHECK_INTERVAL]):
            cls._logger.warn('Not a valid profile as CHECK_INTERVAL is missing or invalid, profile:{}, '
                             'plugin_name:{}, plugin_id:{}'.format(str(sha_profile_pb2), plugin_name, plugin_id))
            return False

        if plugin_id not in cls._plugins_report_to_mp:
            cls._logger.debug('Valid profile:{}, plugin_name:{}, plugin_id:{}'
                              .format(str(sha_profile_pb2), plugin_name, plugin_id))
            return True

        if REPORT_INTERVAL not in content \
                or not cls.is_positive_int(content[REPORT_INTERVAL]):
            cls._logger.warn('Not a valid profile as REPORT_INTERVAL is missing or invalid, profile:{}, '
                             'plugin_name:{}, plugin_id:{}'.format(str(sha_profile_pb2), plugin_name, plugin_id))
            return False

        if SMALLEST_REPORT_INTERVAL_IF_CHANGE not in content \
                or not cls.is_positive_int(content[SMALLEST_REPORT_INTERVAL_IF_CHANGE]):
            cls._logger.warn('Not a valid profile as SMALLEST_REPORT_INTERVAL_IF_CHANGE is missing or invalid, '
                             'profile:{}, plugin_name:{}, plugin_id:{}'.format(str(sha_profile_pb2), plugin_name,
                                                                               plugin_id))
            return False

        cls._logger.debug('Valid profile:{}, plugin_name:{}, plugin_id:{}'
                          .format(str(sha_profile_pb2), plugin_name, plugin_id))
        return True

    @classmethod
    def set(cls, profile_pb2):
        """
        :param profile_pb2: protobuf_msg
        """
        try:
            sha_profile_pb2 = profile_pb2.system_health_profile
            plugin_id = sha_profile_pb2.plugin_id

            # for upgrade case - In Grindcore, predefined plugin does not have plugin_id
            if not plugin_id and sha_profile_pb2.type in cls._type_id_mapper:
                plugin_id = cls._type_id_mapper.get(sha_profile_pb2.type)
                cls._logger.info('upgrade case, get plugin_id:{} for sha_profile_pb2:{}'
                                 .format(plugin_id, str(sha_profile_pb2)))

            priority = profile_pb2.priority
            uuid_msg = profile_pb2.id
            profile_msg_id = convert_to_uuid_str(uuid_msg)
            if not cls.is_valid_profile(plugin_id, sha_profile_pb2):
                return

            content = json.loads(sha_profile_pb2.content) if sha_profile_pb2.content else {}
            profile_collection = cls._profiles.setdefault(plugin_id, ProfileCollection())
            profile_collection.add_or_update(profile_msg_id, priority, sha_profile_pb2.name,
                                             sha_profile_pb2.enabled, content)
        except Exception as e:
            trace = traceback.format_exc()
            cls._logger.error('Failed in set profile_pb2:{}, trace:{}'.format(str(profile_pb2), trace))

    @classmethod
    def clear(cls, profile_pb2):
        plugin_id = profile_pb2.system_health_profile.plugin_id
        sha_profile_pb2 = profile_pb2.system_health_profile

        # for upgrade case - In Grindcore, predefined plugin does not have plugin_id
        if not plugin_id and sha_profile_pb2.type in cls._type_id_mapper:
            plugin_id = cls._type_id_mapper.get(sha_profile_pb2.type)
            cls._logger.info('upgrade case, get plugin_id:{} for sha_profile_pb2:{}'
                             .format(plugin_id, str(sha_profile_pb2)))

        if plugin_id not in cls._profiles:
            cls._logger.info('plugin_id:{} has no profiles applied.'.format(plugin_id, sha_profile_pb2))
            return

        uuid_msg = profile_pb2.id
        profile_msg_id = convert_to_uuid_str(uuid_msg)

        profile_collection = cls._profiles.get(plugin_id)
        profile_collection.delete(profile_msg_id)

        if profile_collection.is_empty():
            cls._profiles.pop(plugin_id)
